@@ -42,7 +42,7 @@ def pandas_df_to_spark_df(spark, pandas_df, save_index = True):
   return spark.createDataFrame( pandas_df, list(pandas_df.columns) )
 
 from pyspark.sql.functions import spark_partition_id
-def spark_df_to_shards_df(spark, spark_df, include_empty_zero_id = True):
+def spark_df_to_shards_df(spark, spark_df, include_empty_zero_id = False):
   df = ( spark_df
           .withColumn("id", spark_partition_id() )
           .groupBy("id")
